/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sighdl.h>

extern mdevice_t *mdevice_g;
extern ptable_t *command_set;

void setup_sighdl(void)
{
	struct sigaction s_action;

	memset(&s_action, 0, sizeof(struct sigaction));

	s_action.sa_handler = shdl_quit;
	sigfillset(&(s_action.sa_mask));
	sigaction(SIGINT, &s_action, NULL);
	sigaction(SIGQUIT, &s_action, NULL);
	sigaction(SIGTERM, &s_action, NULL);

	s_action.sa_handler = SIG_IGN;
	sigemptyset(&(s_action.sa_mask));
	sigaction(SIGPIPE, &s_action, NULL);
}

void shdl_quit(int sig_num)
{
	mnode_t *mnode = NULL;
	mserver_t *mserver = NULL;

	log_msg(INFO_LVL, "Got a signal for exiting (%d)...\n", sig_num);

	/*Aborting DNS resolution*/
	ares_destroy(mdevice_g->dns_req_channel);

#ifdef CARES_HAVE_ARES_LIBRARY_CLEANUP
	ares_library_cleanup();
#endif


	if (mdevice_g->ip_mnodes != NULL)
	{
		h_iter_begin(mdevice_g->ip_mnodes);

		while(h_iter_next(mdevice_g->ip_mnodes))
		{
			if (is_ipv6_unicast(h_iter_getkey(mdevice_g->ip_mnodes), NULL) == 0)
			{
				mnode = h_iter_getvalue(mdevice_g->ip_mnodes);
				remove_mnode(mdevice_g, mnode);
			}
		}

		h_iter_end(mdevice_g->ip_mnodes);
	}

	/*Removing mservers*/
	while (true)
	{
		if ((mserver = mdevice_g->mservers) != NULL)
			remove_mserver(mdevice_g, mserver);
		else
			break;
	}

	/*Unloading plugin here:*/
	deinit_plugin_manager();
	
	/*Ending with mdevice*/
	remove_mdevice();

	pt_destroy(command_set);

	exit(EXIT_SUCCESS);
}
