/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __IO_H__
#define __IO_H__

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/ip6.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <sys/time.h>

#include <vpzone_internal.h>
#include <vpzone.h>
#include <utils.h>
#include <hash.h>
#include <log.h>

/*4 bytes for TUN flags + 40 bytes IPv6 header*/
#define TUN_DEV_MTU 1504

void read_mnode_df(int sock, short event, void *arg);

void manage_packet_df(mnode_t *mnode);

int route_packet_df(uint8_t *buffer, int buff_len, void *arg);

void write_mnode_df(int sock, short event, void *arg);

void read_tun_df(int fd, short event, void *arg);

void write_tun_df(int fd, short event, void *arg);

void remove_mnode_df(mdevice_t *mdevice, mnode_t *mnode);

void remove_mserver_df(mdevice_t *mdevice, mserver_t *mserver);

void remove_mdevice_df(void);

void setup_io(mdevice_t *mdevice);

void poll_io(mdevice_t *mdevice);

#ifdef DEBUG

#define DBG_PACKET_HDR(fct_name, pkt)									\
	{																	\
	char ip_src[INET6_ADDRSTRLEN];										\
	char ip_dst[INET6_ADDRSTRLEN];										\
																		\
	DEBUG_MSG("%s pkt v0x%x - ip_src: %s and ip_dst: %s - len: %d\n",	\
			  fct_name,													\
			  IP6_VER(pkt_hdr),											\
			  inet_ntop(AF_INET6, IP6_SRC(pkt_hdr), ip_src, INET6_ADDRSTRLEN), \
			  inet_ntop(AF_INET6, IP6_DST(pkt_hdr), ip_dst, INET6_ADDRSTRLEN), \
			  IP6_LEN(pkt_hdr) + TUN_PKT_LEN);							\
	}

#else

#define DBG_PACKET_HDR(...) {}

#endif /*DEBUG*/

#endif
