/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __VPZONE_INTERNAL_H__
#define __VPZONE_INTERNAL_H__

#include <config.h>
#include <vpzone.h>

extern handler_t ih[HDL_MAX_HANDLERS];
extern callback_t *icb[CB_MAX_CALLBACKS];
extern ptable_t *connectors;
extern ptable_t *ciphers;
extern ptable_t *authenticators;

#define manage_packet ((manage_packet_f) ih[HDL_MANAGE_PACKET])

#define route_packet ((route_packet_f) ih[HDL_ROUTE_PACKET])

#define read_tun ((read_tun_f) ih[HDL_READ_DEVICE])

#define write_tun ((write_tun_f) ih[HDL_WRITE_DEVICE])

#define remove_mnode ((remove_mnode_f) ih[HDL_REMOVE_MNODE])

#define remove_mserver ((remove_mserver_f) ih[HDL_REMOVE_MSERVER])

#define remove_mdevice ((remove_mdevice_f) ih[HDL_REMOVE_MDEVICE])

#define select_route(ret_var, ...) HDL_RET(ret_var, NULL, select_route_f, ih[HDL_SELECT_ROUTE], __VA_ARGS__)

/*Connector specific*/
#define setup_mserver(m_d, m_s)	((setup_mserver_f) ((connector_t *) connectors->ptable[m_s->connector_id])->setup_mserver_h)(m_d, m_s)
#define close_mserver(m_d, m_s) ((setup_mserver_f) ((connector_t *) connectors->ptable[m_s->connector_id])->close_mserver_h)(m_d, m_s)
#define connect_mnode(m_n) ((connect_mnode_f) ((connector_t *) connectors->ptable[m_n->connector_id])->connect_mnode_h)(m_n)
#define disconnect_mnode(m_n) ((disconnect_mnode_f) ((connector_t *) connectors->ptable[m_n->connector_id])->disconnect_mnode_h)(m_n)

/*Cipher specific*/
#define setup_mnode_server_cipher(m_n)									\
	if (((cipher_t *) ciphers->ptable[m_n->cipher_id])->setup_mnode_server_cipher_h != NULL) \
		((setup_mnode_server_cipher_f)									\
		 ((cipher_t *) ciphers->ptable[m_n->cipher_id])->setup_mnode_server_cipher_h)(m_n)

#define setup_mnode_client_cipher(m_n)									\
	if (((cipher_t *) ciphers->ptable[m_n->cipher_id])->setup_mnode_client_cipher_h != NULL) \
		((setup_mnode_client_cipher_f)									\
		 ((cipher_t *) ciphers->ptable[m_n->cipher_id])->setup_mnode_client_cipher_h)(m_n)

#define deinit_mnode_server_cipher(m_n)									\
	if (((cipher_t *) ciphers->ptable[m_n->cipher_id])->deinit_mnode_server_cipher_h != NULL) \
		((deinit_mnode_server_cipher_f)									\
		 ((cipher_t *) ciphers->ptable[m_n->cipher_id])->deinit_mnode_server_cipher_h)(m_n)

#define deinit_mnode_client_cipher(m_n)									\
	if (((cipher_t *) ciphers->ptable[m_n->cipher_id])->deinit_mnode_client_cipher_h != NULL) \
		((deinit_mnode_client_cipher_f)									\
		 ((cipher_t *) ciphers->ptable[m_n->cipher_id])->deinit_mnode_client_cipher_h)(m_n)

#define recv_mnode(s, e, m_n)							\
	((recv_mnode_f)										\
	 ((cipher_t *) ciphers->ptable[m_n->cipher_id])->recv_mnode_h)(s, e, m_n)

#define recv_mnode_h(m_n)							\
	(recv_mnode_f)									\
	((cipher_t *) ciphers->ptable[m_n->cipher_id])->recv_mnode_h

#define send_mnode(s, e, m_n)							\
	((send_mnode_f)										\
	 ((cipher_t *) ciphers->ptable[m_n->cipher_id])->send_mnode_h)(s, e, m_n)

#define send_mnode_h(m_n)								\
	(send_mnode_f)										\
	((cipher_t *) ciphers->ptable[m_n->cipher_id])->send_mnode_h

/*Authenticators specific*/
#define setup_mnode_server_authentification(m_n)						\
	if (((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->setup_mnode_server_authentification_h != NULL) \
		((setup_mnode_server_authentification_f)						\
		 ((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->setup_mnode_server_authentification_h)(m_n)

#define setup_mnode_client_authentification(m_n)						\
	if (((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->setup_mnode_client_authentification_h != NULL) \
		((setup_mnode_client_authentification_f)						\
		 ((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->setup_mnode_client_authentification_h)(m_n)

#define deinit_mnode_server_authentification(m_n)						\
	if (((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->deinit_mnode_server_authentification_h != NULL) \
		((deinit_mnode_server_authentification_f)						\
		 ((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->deinit_mnode_server_authentification_h)(m_n)

#define deinit_mnode_client_authentification(m_n)						\
	if (((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->deinit_mnode_client_authentification_h != NULL) \
		((deinit_mnode_client_authentification_f)						\
		 ((authenticator_t *) authenticators->ptable[m_n->authenticator_id])->deinit_mnode_client_authentification_h)(m_n)

/*Miscellaneous*/
#define manage_internal_packet(ret_var, ...) CB_RET(			\
	manage_internal_packet_f, icb[CB_MANAGE_INTERNAL_PACKET],	\
	ret_var, 0, __VA_ARGS__)

#define post_accept(...) CB_VOID(post_accept_f,					\
								 icb[CB_POST_ACCEPT], __VA_ARGS__)

#define post_connect(...) CB_VOID(post_connect_f,						\
								  icb[CB_POST_CONNECT], __VA_ARGS__)

#define pre_disconnect(...) CB_VOID(pre_disconnect_f,					\
									icb[CB_PRE_DISCONNECT], __VA_ARGS__)

#define init_mnode_extent(...) CB_VOID(init_mnode_extent_f,				\
									   icb[CB_INIT_MNODE_EXTENT], __VA_ARGS__)

#define remove_mnode_extent(...) CB_VOID(remove_mnode_extent_f,			\
										 icb[CB_REMOVE_MNODE_EXTENT], __VA_ARGS__)

#define init_mserver_extent(...) CB_VOID(init_mserver_extent_f,			\
										 icb[CB_INIT_MSERVER_EXTENT], __VA_ARGS__)

#define remove_mserver_extent(...) CB_VOID(remove_mserver_extent_f,		\
										   icb[CB_REMOVE_MSERVER_EXTENT], __VA_ARGS__)

/*IPv6 header parsing helpers*/

#define IPV6_VERSION 0x6

#define IP6_VER(p_h) (ntohs(*((uint16_t *) p_h)) >> 12)

#define IP6_SRC(p_h) (p_h+8)

#define IP6_DST(p_h) (p_h+24)

#define IP6_LEN(p_h) ntohs(*((uint16_t *) (p_h+4)))

#endif
