/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <centnet.h>

PLUGIN_DEPENDANCIES(NULL);

PLUGIN_GBL_VARS;

plugin_type_t init(int plugin_id, in_plugin_handlers_t *iph)
{
	PLUGIN_SET_VARS(plugin_id, iph);

	set_handler(HDL_ROUTE_PACKET, route_packet_centnet);

	log_msg(INFO_LVL, "Loaded CENTNET router plugin\n");

	return PLUGIN_DEFAULT;
}

int unload(void)
{
	unset_handler(HDL_ROUTE_PACKET);

	return 0;
}

int route_packet_centnet(uint8_t *buffer, int buff_len, void *arg)
{
	int pkt_len = 0;
	struct ip6_hdr *pkt_hdr = (struct ip6_hdr*) arg;
	mnode_t *mnode = NULL;

	h_iter_begin(mdevice_g->ip_mnodes);

	if (h_iter_next(mdevice_g->ip_mnodes))
		mnode = h_iter_getvalue(mdevice_g->ip_mnodes);

	h_iter_end(mdevice_g->ip_mnodes);

	if (mnode == NULL)
	{
		log_msg(WARNING_LVL, "Could not proceed with sending packet to central server\n");
		return -1;
	}

	pkt_len = (int) ntohs(pkt_hdr->ip6_plen) + TUN_PKT_LEN;

	if (pkt_len > buff_len)
		return -1;

	memcpy(mnode->w_buffer + mnode->w_bytes, buffer, pkt_len);
	mnode->w_bytes += pkt_len;

	send_mnode(mnode->sock, EV_WRITE, mnode);

	return pkt_len;
}
